@@ -5,6 +5,21 @@ from primestg.report.base import (
 )
 from primestg.message import MessageS
 
+SUPPORTED_REPORTS = ['S02', 'S04', 'S05', 'S06', 'S09', 'S12', 'S13', 'S15',
+                     'S17', 'S24', 'S27']
+
+
+def is_supported(report_code):
+    return report_code in SUPPORTED_REPORTS
+
+
+def get_integer_value(param):
+    try:
+        result = int(param)
+    except ValueError as e:
+        result = 0
+    return result
+
 
 class MeasureS02(MeasureActiveReactive):
     """
@@ -18,14 +33,20 @@ class MeasureS02(MeasureActiveReactive):
 
         :return: a dict with a set of measures of report S02
         """
-        values = self.active_reactive(self.objectified, '')
-        values.update(
-            {
-                'timestamp': self._get_timestamp('Fh'),
-                'season': self.objectified.get('Fh')[-1:],
-                'bc': self.objectified.get('Bc')
-            }
-        )
+        values = {}
+        try:
+            values = self.active_reactive(self.objectified, '')
+            values.update(
+                {
+                    'timestamp': self._get_timestamp('Fh'),
+                    'season': self.objectified.get('Fh')[-1:],
+                    'bc': self.objectified.get('Bc')
+                }
+            )
+        except Exception as e:
+            self._warnings.append('ERROR: Thrown exception: {}'.format(e))
+            return []
+
         return [values]
 
 
@@ -42,24 +63,27 @@ class MeasureS04(MeasureActiveReactive):
         :return: a dict with a set of measures of report S04
         """
         values = []
-        common_values = {
-            'type': 'month',
-            'date_begin': self._get_timestamp('Fhi'),
-            'date_end': self._get_timestamp('Fhf'),
-            'contract': int(self.objectified.get('Ctr')),
-            'period': int(self.objectified.get('Pt')),
-            'max': int(self.objectified.get('Mx')),
-            'date_max': self._get_timestamp('Fx')
-        }
-        for s04_values in self.objectified.Value:
-            v = common_values.copy()
-            if s04_values.get('AIa'):
-                measure_type = 'a'
-            else:
-                measure_type = 'i'
-            v.update(self.active_reactive(s04_values, measure_type))
-            v['value'] = measure_type
-            values.append(v)
+        try:
+            common_values = {
+                'type': 'month',
+                'date_begin': self._get_timestamp('Fhi'),
+                'date_end': self._get_timestamp('Fhf'),
+                'contract': int(self.objectified.get('Ctr')),
+                'period': int(self.objectified.get('Pt')),
+                'max': int(self.objectified.get('Mx')),
+                'date_max': self._get_timestamp('Fx')
+            }
+            for s04_values in self.objectified.Value:
+                v = common_values.copy()
+                if s04_values.get('AIa'):
+                    measure_type = 'a'
+                else:
+                    measure_type = 'i'
+                v.update(self.active_reactive(s04_values, measure_type))
+                v['value'] = measure_type
+                values.append(v)
+        except Exception as e:
+            self._warnings.append('ERROR: Thrown exception: {}'.format(e))
         return values
 
 
@@ -76,19 +100,58 @@ class MeasureS05(MeasureActiveReactive):
         :return: a dict with a set of measures of report S05
         """
         values = []
-        timestamp = self._get_timestamp('Fh')
-        v = {
-            'type': 'day',
-            'value': 'a',
-            'date_begin': timestamp,
-            'date_end': timestamp,
-            'contract': int(self.objectified.get('Ctr')),
-            'period': int(self.objectified.get('Pt')),
-        }
+        try:
+            timestamp = self._get_timestamp('Fh')
+            v = {
+                'type': 'day',
+                'value': 'a',
+                'date_begin': timestamp,
+                'date_end': timestamp,
+                'contract': int(self.objectified.get('Ctr')),
+                'period': int(self.objectified.get('Pt')),
+            }
 
-        for s05_values in self.objectified.Value:
-            v.update(self.active_reactive(s05_values, 'a'))
-            values.append(v)
+            for s05_values in self.objectified.Value:
+                v.update(self.active_reactive(s05_values, 'a'))
+                values.append(v)
+        except Exception as e:
+            self._warnings.append('ERROR: Thrown exception: {}'.format(e))
+
+        return values
+
+
+class MeasureS27(MeasureActiveReactive):
+    """
+    Class for a set of measures of report S27.
+    """
+
+    @property
+    def values(self):
+        """
+        Set of measures of report S27.
+
+        :return: a dict with a set of measures of report S27
+        """
+        values = []
+        try:
+            timestamp = self._get_timestamp('Fh')
+            timestamp_max = self._get_timestamp('Fx')
+            v = {
+                'type': 'manual',
+                'value': 'a',
+                'date_begin': timestamp,
+                'date_end': timestamp,
+                'contract': int(self.objectified.get('Ctr')),
+                'period': int(self.objectified.get('Pt')),
+                'max': int(self.objectified.get('Mx')),
+                'date_max': timestamp_max
+            }
+
+            for s27_values in self.objectified.Value:
+                v.update(self.active_reactive(s27_values, 'a'))
+                values.append(v)
+        except Exception as e:
+            self._warnings.append('ERROR: Thrown exception: {}'.format(e))
 
         return values
 
@@ -106,23 +169,27 @@ class MeasureEvents(Measure):
         :return: a dict with a set of measures of report S09
         """
         values = []
-        timestamp = self._get_timestamp('Fh')
-        v = {
-            'timestamp': timestamp,
-            'event_group': int(self.objectified.get('Et')),
-            'season': self.objectified.get('Fh')[-1:],
-            'event_code': int(self.objectified.get('C')),
-        }
-        data = ''
-        d1s = ['D1: {}'.format(d)
-               for d in getattr(self.objectified, 'D1', [])]
-        d2s = ['D2: {}'.format(d)
-               for d in getattr(self.objectified, 'D2', [])]
-        data = '\n'.join(d1s + d2s)
-        if data:
-            v.update({'data': data})
+        try:
+            timestamp = self._get_timestamp('Fh')
+            v = {
+                'timestamp': timestamp,
+                'event_group': int(self.objectified.get('Et')),
+                'season': self.objectified.get('Fh')[-1:],
+                'event_code': int(self.objectified.get('C')),
+            }
+            data = ''
+            d1s = ['D1: {}'.format(d)
+                   for d in getattr(self.objectified, 'D1', [])]
+            d2s = ['D2: {}'.format(d)
+                   for d in getattr(self.objectified, 'D2', [])]
+            data = '\n'.join(d1s + d2s)
+            if data:
+                v.update({'data': data})
 
-        values.append(v)
+            values.append(v)
+        except Exception as e:
+            self._warnings.append('ERROR: Reading a meter event. Thrown '
+                                  'exception: {}'.format(e))
 
         return values
 
@@ -216,42 +283,48 @@ class ParameterS06(Parameter):
 
         :return: a dict with a set of parameters of report S06
         """
-        get = self.objectified.get
+        values = {}
+        try:
+            get = self.objectified.get
 
-        values = {
-            'request_id': self.request_id,
-            'version': self.report_version,
-            'concentrator': self.concentrator_name,
-            'meter': self.meter_name,
-            'timestamp': self._get_timestamp('Fh'),
-            'season': get('Fh')[-1:],
-            'serial_number': get('NS'),
-            'manufacturer': get('Fab'),
-            'model_type': get('Mod'),
-            'manufacturing_year': int(get('Af')),
-            'equipment_type': get('Te'),
-            'firmware_version': get('Vf'),
-            'prime_firmware_version': get('VPrime'),
-            'protocol': get('Pro'),
-            'id_multicast': get('Idm'),
-            'mac': get('Mac'),
-            'primary_voltage': int(get('Tp')),
-            'secondary_voltage': int(get('Ts')),
-            'primary_current': int(get('Ip')),
-            'secondary_current': int(get('Is')),
-            'time_threshold_voltage_sags': int(get('Usag')),
-            'time_threshold_voltage_swells': int(get('Uswell')),
-            'load_profile_period': int(get('Per')),
-            'demand_close_contracted_power': get('Dctcp'),
-            'reference_voltage': int(get('Vr')),
-            'long_power_failure_threshold': int(get('Ut')),
-            'voltage_sag_threshold': get('UsubT'),
-            'voltage_swell_threshold': get('UsobT'),
-            'voltage_cut-off_threshold': get('UcorteT'),
-            'automatic_monthly_billing': self.get_boolean('AutMothBill'),
-            'scroll_display_mode': get('ScrollDispMode'),
-            'time_scroll_display': int(get('ScrollDispTime'))
-        }
+            values = {
+                'request_id': self.request_id,
+                'version': self.report_version,
+                'concentrator': self.concentrator_name,
+                'meter': self.meter_name,
+                'timestamp': self._get_timestamp('Fh'),
+                'season': get('Fh')[-1:],
+                'serial_number': get('NS'),
+                'manufacturer': get('Fab'),
+                'model_type': get('Mod'),
+                'manufacturing_year': get_integer_value(get('Af')),
+                'equipment_type': get('Te'),
+                'firmware_version': get('Vf'),
+                'prime_firmware_version': get('VPrime'),
+                'protocol': get('Pro'),
+                'id_multicast': get('Idm'),
+                'mac': get('Mac'),
+                'primary_voltage': get_integer_value(get('Tp')),
+                'secondary_voltage': get_integer_value(get('Ts')),
+                'primary_current': get_integer_value(get('Ip')),
+                'secondary_current': get_integer_value(get('Is')),
+                'time_threshold_voltage_sags': get_integer_value(get('Usag')),
+                'time_threshold_voltage_swells': get_integer_value(get('Uswell')),
+                'load_profile_period': get_integer_value(get('Per')),
+                'demand_close_contracted_power': get('Dctcp'),
+                'reference_voltage': get_integer_value(get('Vr')),
+                'long_power_failure_threshold': get_integer_value(get('Ut')),
+                'voltage_sag_threshold': get('UsubT'),
+                'voltage_swell_threshold': get('UsobT'),
+                'voltage_cut-off_threshold': get('UcorteT'),
+                'automatic_monthly_billing': self.get_boolean('AutMothBill'),
+                'scroll_display_mode': get('ScrollDispMode'),
+                'time_scroll_display': get_integer_value(get('ScrollDispTime'))
+            }
+        except Exception as e:
+            self._warnings.append('ERROR: Cnc({}), Meter({}). Thrown '
+                                  'exception: {}'.format(self.concentrator_name,
+                                                         self.meter_name, e))
         return values
 
 
@@ -267,118 +340,210 @@ class ParameterS12(Parameter):
 
         :return: a dict with a set of parameters of report S12
         """
-        get = self.objectified.get
+        values = {}
 
-        if self.report_version == '3.1c':
-            fwmtup_timeout_key = 'TimeOutMeterFwU'
-        else:
-            fwmtup_timeout_key = 'TimeOutPrimeFwU'
+        try:
+            get = self.objectified.get
 
-        fwmtup_timeout = self.to_integer(get(fwmtup_timeout_key))
+            if self.report_version == '3.1c':
+                fwmtup_timeout_key = 'TimeOutMeterFwU'
+            else:
+                fwmtup_timeout_key = 'TimeOutPrimeFwU'
 
-        # Ormazabal Current concentrators returns the IPftp1 field
-        if 'IPftp' in self.objectified.keys():
-            rpt_ftp_ip_address_key = 'IPftp'
-        else:
-            rpt_ftp_ip_address_key = 'IPftp1'
+            fwmtup_timeout = self.to_integer(get(fwmtup_timeout_key))
 
-        rpt_ftp_ip_address = get(rpt_ftp_ip_address_key)
+            # Ormazabal Current concentrators returns the IPftp1 field
+            if 'IPftp' in self.objectified.keys():
+                rpt_ftp_ip_address_key = 'IPftp'
+            else:
+                rpt_ftp_ip_address_key = 'IPftp1'
 
-        ntp_max_deviation = self.filter_integer(get('NTPMaxDeviation'))
-        session_timeout = self.filter_integer(get('AccInacTimeout'))
-        max_sessions = self.filter_integer(get('AccSimulMax'))
+            rpt_ftp_ip_address = get(rpt_ftp_ip_address_key)
 
-        values = {
-            'date': self._get_timestamp('Fh'),
-            'model': get('Mod'),
-            'mf_year': get('Af'),
-            'type': get('Te'),
-            'w_password': get('DCPwdAdm'),
-            'r_password': get('DCPwdRead'),
-            'fw_version': get('Vf'),
-            'fw_comm_version': get('VfComm'),
-            'protocol': get('Pro'),
-            'communication': get('Com'),
-            'battery_mon': get('Bat'),
-            'ip_address': get('ipCom'),
-            'dc_ws_port': get('PortWS'),
-            'ip_mask': get('ipMask'),
-            'ip_gtw': get('ipGtw'),
-            'dhcp': self.get_boolean('ipDhcp'),
-            'slave1': get('Slave1'),
-            'slave2': get('Slave2'),
-            'slave3': get('Slave3'),
-            'local_ip_address': get('ipLoc'),
-            'local_ip_mask': get('ipMaskLoc'),
-            'plc_mac': get('Macplc'),
-            'serial_port_speed': get('Pse'),
-            'priority': self.get_boolean('Priority'),
-            'stg_ws_ip_address': get('IPstg'),
-            'stg_ws_password': get('stgPwd'),
-            'ntp_ip_address': get('IPNTP'),
-            'rpt_ftp_ip_address': rpt_ftp_ip_address,
-            'rpt_ftp_user': get('FTPUserReport'),
-            'rpt_ftp_password': get('FTPPwdReport'),
-            'fwdcup_ftp_ip_address': get('IPftpDCUpg'),
-            'fwdcup_ftp_user': get('UserftpDCUpg'),
-            'fwdcup_ftp_password': get('PwdftpDCUpg'),
-            'fwmtup_ftp_ip_address': get('IPftpMeterUpg'),
-            'fwmtup_ftp_user': get('UserftpMeterUpg'),
-            'fwmtup_ftp_password': get('UserftpMeterUpg'),
-            'retries': int(get('RetryFtp')),
-            'time_btw_retries': int(get('TimeBetwFtp')),
-            'cycle_ftp_ip_address': get('IPftpCycles'),
-            'cycle_ftp_user': get('UserftpCycles'),
-            'cycle_ftp_password': get('PwdftpCycles'),
-            'cycle_ftp_dir': get('DestDirCycles'),
-            'sync_meter': self.get_boolean('SyncMeter'),
-            'fwmtup_timeout': fwmtup_timeout,
-            'max_time_deviation': int(get('TimeDevOver')),
-            'min_time_deviation': int(get('TimeDev')),
-            'reset_msg': self.get_boolean('ResetMsg'),
-            'rpt_meter_limit': int(get('NumMeters')),
-            'rpt_time_limit': int(get('TimeSendReq')),
-            'disconn_time': int(get('TimeDisconMeter')),
-            'disconn_retries': int(get('RetryDisconMeter')),
-            'disconn_retry_interval': int(get('TimeRetryInterval')),
-            'meter_reg_data': get('MeterRegData'),
-            'report_format': get('ReportFormat'),
-            's26_content': get('S26Content'),
-            'values_check_delay': int(get('ValuesCheckDelay')),
-            'max_order_outdate': self.to_integer(get('MaxOrderOutdate')),
-            'restart_delay': self.to_integer(get('TimeDelayRestart')),
-            'ntp_max_deviation': ntp_max_deviation,
-            'session_timeout': session_timeout,
-            'max_sessions':  max_sessions
-        }
-        if hasattr(self.objectified, 'TP'):
-            tasks = []
-            for task in self.objectified.TP:
-                task_values = {
-                    'name': task.get('TpTar'),
-                    'priority': int(task.get('TpPrio')),
-                    'date_from': self._get_timestamp('TpHi', element=task),
-                    'periodicity': task.get('TpPer'),
-                    'complete': self.get_boolean('TpCompl', element=task),
-                    'meters': task.get('TpMet'),
-                }
-                task_data_values = []
-                if getattr(task, 'TpPro', None):
-                    for task_data in task.TpPro:
-                        task_data_value = {
-                            'request': task_data.get('TpReq'),
-                            'stg_send':
-                                self.get_boolean('TpSend', element=task_data),
-                            'store':
-                                self.get_boolean('TpStore', element=task_data),
-                            'attributes': task_data.get('TpAttr'),
-                        }
-                        task_data_values.append(task_data_value)
-                task_values['task_data'] = task_data_values
-                tasks.append(task_values)
-            values['tasks'] = tasks
-        else:
-            values['tasks'] = []
+            ntp_max_deviation = self.filter_integer(get('NTPMaxDeviation'))
+            session_timeout = self.filter_integer(get('AccInacTimeout'))
+            max_sessions = self.filter_integer(get('AccSimulMax'))
+
+            values = {
+                'date': self._get_timestamp('Fh'),
+                'model': get('Mod'),
+                'mf_year': get('Af'),
+                'type': get('Te'),
+                'w_password': get('DCPwdAdm'),
+                'r_password': get('DCPwdRead'),
+                'fw_version': get('Vf'),
+                'fw_comm_version': get('VfComm'),
+                'protocol': get('Pro'),
+                'communication': get('Com'),
+                'battery_mon': get('Bat'),
+                'ip_address': get('ipCom'),
+                'dc_ws_port': get('PortWS'),
+                'ip_mask': get('ipMask'),
+                'ip_gtw': get('ipGtw'),
+                'dhcp': self.get_boolean('ipDhcp'),
+                'slave1': get('Slave1'),
+                'slave2': get('Slave2'),
+                'slave3': get('Slave3'),
+                'local_ip_address': get('ipLoc'),
+                'local_ip_mask': get('ipMaskLoc'),
+                'plc_mac': get('Macplc'),
+                'serial_port_speed': get('Pse'),
+                'priority': self.get_boolean('Priority'),
+                'stg_ws_ip_address': get('IPstg'),
+                'stg_ws_password': get('stgPwd'),
+                'ntp_ip_address': get('IPNTP'),
+                'rpt_ftp_ip_address': rpt_ftp_ip_address,
+                'rpt_ftp_user': get('FTPUserReport'),
+                'rpt_ftp_password': get('FTPPwdReport'),
+                'fwdcup_ftp_ip_address': get('IPftpDCUpg'),
+                'fwdcup_ftp_user': get('UserftpDCUpg'),
+                'fwdcup_ftp_password': get('PwdftpDCUpg'),
+                'fwmtup_ftp_ip_address': get('IPftpMeterUpg'),
+                'fwmtup_ftp_user': get('UserftpMeterUpg'),
+                'fwmtup_ftp_password': get('UserftpMeterUpg'),
+                'retries': int(get('RetryFtp')),
+                'time_btw_retries': int(get('TimeBetwFtp')),
+                'cycle_ftp_ip_address': get('IPftpCycles'),
+                'cycle_ftp_user': get('UserftpCycles'),
+                'cycle_ftp_password': get('PwdftpCycles'),
+                'cycle_ftp_dir': get('DestDirCycles'),
+                'sync_meter': self.get_boolean('SyncMeter'),
+                'fwmtup_timeout': fwmtup_timeout,
+                'max_time_deviation': int(get('TimeDevOver')),
+                'min_time_deviation': int(get('TimeDev')),
+                'reset_msg': self.get_boolean('ResetMsg'),
+                'rpt_meter_limit': int(get('NumMeters')),
+                'rpt_time_limit': int(get('TimeSendReq')),
+                'disconn_time': int(get('TimeDisconMeter')),
+                'disconn_retries': int(get('RetryDisconMeter')),
+                'disconn_retry_interval': int(get('TimeRetryInterval')),
+                'meter_reg_data': get('MeterRegData'),
+                'report_format': get('ReportFormat'),
+                's26_content': get('S26Content'),
+                'values_check_delay': int(get('ValuesCheckDelay')),
+                'max_order_outdate': self.to_integer(get('MaxOrderOutdate')),
+                'restart_delay': self.to_integer(get('TimeDelayRestart')),
+                'ntp_max_deviation': ntp_max_deviation,
+                'session_timeout': session_timeout,
+                'max_sessions':  max_sessions
+            }
+            if hasattr(self.objectified, 'TP'):
+                tasks = []
+                for task in self.objectified.TP:
+                    task_values = {
+                        'name': task.get('TpTar'),
+                        'priority': int(task.get('TpPrio')),
+                        'date_from': self._get_timestamp('TpHi', element=task),
+                        'periodicity': task.get('TpPer'),
+                        'complete': self.get_boolean('TpCompl', element=task),
+                        'meters': task.get('TpMet'),
+                    }
+                    task_data_values = []
+                    if getattr(task, 'TpPro', None) is not None:
+                        for task_data in task.TpPro:
+                            task_data_value = {
+                                'request': task_data.get('TpReq'),
+                                'stg_send':
+                                    self.get_boolean('TpSend', element=task_data),
+                                'store':
+                                    self.get_boolean('TpStore', element=task_data),
+                                'attributes': task_data.get('TpAttr'),
+                            }
+                            task_data_values.append(task_data_value)
+                    task_values['task_data'] = task_data_values
+                    tasks.append(task_values)
+                values['tasks'] = tasks
+            else:
+                values['tasks'] = []
+        except Exception as e:
+            self._warnings.append('ERROR: Reading S12 report. Thrown '
+                                  'exception: {}'.format(e))
+        return values
+
+
+class ParameterS24(Parameter):
+    """
+    Class for a set of parameters of report S24.
+    """
+
+    def __init__(
+            self,
+            objectified_parameter,
+            report_version,
+            concentrator_name,
+            request_id
+    ):
+        """
+        Create a ParameterS24 object.
+
+        :param objectified_parameter: an lxml.objectify.StringElement \
+            representing a set of parameters
+        :return: a Measure object
+        """
+        super(ParameterS24, self).__init__(
+            objectified_parameter,
+            report_version
+        )
+        self.concentrator_name = concentrator_name
+        self.request_id = request_id
+
+    @property
+    def concentrator_name(self):
+        """
+        A string with the concentrator name.
+
+        :return: a string with the concentrator name
+        """
+        return self._concentrator_name
+
+    @concentrator_name.setter
+    def concentrator_name(self, value):
+        """
+        Stores a string with the concentrator name.
+
+        :param value: a string with the concentrator name
+        """
+        self._concentrator_name = value
+
+    @property
+    def request_id(self):
+        """
+        The request identification.
+
+        :return: a string with the request identification
+        """
+        return self._request_id
+
+    @request_id.setter
+    def request_id(self, value):
+        """
+        Stores the request identification.
+
+        :param value: a string with the version of the report
+        """
+        self._request_id = value
+
+    @property
+    def values(self):
+        """
+        Set of parameters of report S24.
+
+        :return: a dict with a set of parameters of report S24
+        """
+        values = {}
+
+        try:
+            timestamp = self._get_timestamp('Fh')
+            values = {
+                'timestamp': timestamp,
+                'season': self.objectified.get('Fh')[-1:],
+                'cnc_name': self.concentrator_name,
+                'meters': []
+            }
+            for s24_meters in self.objectified.Meter:
+                values['meters'].append(self.meter_availability(s24_meters))
+        except Exception as e:
+            self._warnings.append('ERROR: Thrown exception: {}'.format(e))
         return values
 
 
@@ -451,23 +616,29 @@ class ParameterConcentratorEvents(Parameter):
 
         :return: a dict with a set of parameters of report S17
         """
-        get = self.objectified.get
-        values = {
-            'name': self.concentrator_name,
-            'event_code': int(get('C')),
-            'season': get('Fh')[-1:],
-            'timestamp': self._get_timestamp('Fh'),
-            'event_group': int(get('Et'))
-        }
+        values = []
+        try:
+            get = self.objectified.get
+            values = {
+                'name': self.concentrator_name,
+                'event_code': int(get('C')),
+                'season': get('Fh')[-1:],
+                'timestamp': self._get_timestamp('Fh'),
+                'event_group': int(get('Et'))
+            }
 
-        data = ''
-        d1s = ['D1: {}'.format(d)
-               for d in getattr(self.objectified, 'D1', [])]
-        d2s = ['D2: {}'.format(d)
-               for d in getattr(self.objectified, 'D2', [])]
-        data = '\n'.join(d1s + d2s)
-        if data:
-            values.update({'data': data})
+            data = ''
+            d1s = ['D1: {}'.format(d)
+                   for d in getattr(self.objectified, 'D1', [])]
+            d2s = ['D2: {}'.format(d)
+                   for d in getattr(self.objectified, 'D2', [])]
+            data = '\n'.join(d1s + d2s)
+            if data:
+                values.update({'data': data})
+        except Exception as e:
+            self._warnings.append('ERROR: Reading a concentrator event. Thrown '
+                                  'exception: {}'.format(e))
+            return []
 
         return values
 
@@ -558,6 +729,31 @@ class MeterS05(MeterWithMagnitude):
         return MeasureS05
 
 
+class MeterS27(MeterWithMagnitude):
+    """
+    Class for a meter of report S27.
+    """
+
+    @property
+    def report_type(self):
+        """
+        The type of report for report S27.
+
+        :return: a string with 'S27'
+        """
+
+        return 'S27'
+
+    @property
+    def measure_class(self):
+        """
+        The class used to instance measure sets for report S27.
+
+        :return: a class to instance measure sets of report S27
+        """
+        return MeasureS27
+
+
 class MeterS06(MeterWithMagnitude):
     """
     Class for a meter of report S06.
@@ -651,8 +847,22 @@ class MeterS06(MeterWithMagnitude):
         values = []
         for parameter in self.parameters:
             values.append(parameter.values)
+            if parameter.warnings:
+                if self._warnings.get(self.name, False):
+                    self._warnings[self.name].extend(parameter.warnings)
+                else:
+                    self._warnings.update({self.name: parameter.warnings})
         return values
 
+    @property
+    def warnings(self):
+        """
+        Warnings of this meter.
+
+        :return: a list with the errors found reading the meter
+        """
+        return self._warnings
+
 
 class MeterS09(MeterWithConcentratorName):
     """
@@ -749,6 +959,21 @@ class ConcentratorS05(ConcentratorWithMetersWithConcentratorName):
         return MeterS05
 
 
+class ConcentratorS27(ConcentratorWithMetersWithConcentratorName):
+    """
+    Class for a concentrator of report S27.
+    """
+
+    @property
+    def meter_class(self):
+        """
+        The class used to instance meters for report S27.
+
+        :return: a class to instance meters of report S27
+        """
+        return MeterS27
+
+
 class ConcentratorS06(ConcentratorWithMetersWithConcentratorName):
     """
     Class for a concentrator of report S06.
@@ -821,6 +1046,8 @@ class ConcentratorS06(ConcentratorWithMetersWithConcentratorName):
                     self.report_version,
                     self.request_id
                 ))
+            for meter in meters:
+                self._warnings.append(meter.warnings)
         return meters
 
 
@@ -896,6 +1123,7 @@ class ConcentratorS12(Concentrator):
         values = []
         for parameter in self.parameters:
             values.append(parameter.values)
+            self._warnings.extend(parameter.warnings)
         return values
 
 
@@ -978,9 +1206,11 @@ class ConcentratorEvents(Concentrator):
         :return: a list with the values of the meters
         """
         values = []
-
         for parameter in self.parameters:
-            values.append(parameter.values)
+            if parameter.values:
+                values.append(parameter.values)
+            if parameter.warnings:
+                self._warnings.extend(parameter.warnings)
         return values
 
 
@@ -1046,6 +1276,52 @@ class ConcentratorS17(ConcentratorEvents):
         return parameters
 
 
+class ConcentratorS24(Concentrator):
+    """
+        Class for a concentrator of report S24.
+    """
+
+    def __init__(self, objectified_concentrator, report_version, request_id,
+                 report_type):
+        """
+
+        """
+        super(ConcentratorS24, self).__init__(objectified_concentrator)
+        self.report_version = report_version
+        self.request_id = request_id
+        self.report_type = report_type
+
+    @property
+    def parameters(self):
+        """
+        Parameter set objects of this concentrator.
+
+        :return: a list of parameter set objects
+        """
+        parameters = []
+        if getattr(self.objectified, 'S24', None) is not None:
+            for parameter in self.objectified.S24:
+                parameters.append(ParameterS24(
+                    parameter,
+                    self.report_version,
+                    self.name,
+                    self.request_id))
+        return parameters
+
+    @property
+    def values(self):
+        """
+        Values of the set of parameters of this concentrator.
+
+        :return: a list with the values of the meters
+        """
+        values = []
+        for parameter in self.parameters:
+            values.append(parameter.values)
+            self._warnings.extend(parameter.warnings)
+        return values
+
+
 class Report(object):
     """
     Report class to process MessageS
@@ -1078,7 +1354,7 @@ class Report(object):
         :param value: a file object or a string with the XML or a MessageS \
             object
         """
-        if isinstance(value, (file, basestring)):
+        if hasattr(value, 'read') or isinstance(value, str):
             message = MessageS(value)
         elif isinstance(value, MessageS):
             message = value
@@ -1171,7 +1447,20 @@ class Report(object):
                     self.request_id,
                     self.report_type
                 ]
-            }
+            },
+            'S24': {
+                'class': ConcentratorS24,
+                'args': [
+                    objectified_concentrator,
+                    self.report_version,
+                    self.request_id,
+                    self.report_type
+                ]
+            },
+            'S27': {
+                'class': ConcentratorS27,
+                'args': [objectified_concentrator]
+            },
         }
 
         if self.report_type not in report_type_class:
@@ -1183,6 +1472,10 @@ class Report(object):
         concentrator = concentrator_class(*concentrator_args)
         return concentrator
 
+    @property
+    def supported(self):
+        return is_supported(self.report_type)
+
     @property
     def concentrators(self):
         """
